#!/usr/bin/env python

"""
Moin wiki table parser.

Copyright (C) 2017 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.parsers.common import get_patterns
from moinformat.parsers.moin import MoinParser
from moinformat.tree import Table, TableAttrs, TableCell, TableRow, Text



# Parser functionality.

class TableParser(MoinParser):

    "A parser for improved table syntax."

    # Principal parser methods.

    def parse_region_content(self, items, region):

        "Parse the data provided by 'items' to populate the given 'region'."

        self.set_region(items, region)
        self.parse_table_region()

    def parse_table_region(self):

        # Start to populate table rows.

        cell = TableCell([])
        row = TableRow([cell])
        table = Table([row])
        self.append_node(self.region, table)

        while True:
            self.parse_region_details(cell, self.table_region_pattern_names)

            # Detect the end of the table.

            if self.read_matching() == "regionend":
                break

            if self.read_matching() == "columnsep":
                cell = TableCell([])
                row.append(cell)

            elif self.read_matching() == "rowsep":
                row = TableRow([])
                table.append(row)
                cell = TableCell([])
                row.append(cell)

    # Parser handler methods.

    def parse_continuation(self, cell):
        pass

    def parse_table_end(self, cell):

        "Handle the end of a region within 'cell'."

        feature = self.read_match()
        if self.region.have_end(feature):
            raise StopIteration
        else:
            cell.append_inline(Text(feature))

    # Regular expressions.

    syntax = {}
    syntax.update(MoinParser.syntax)
    syntax.update({
        # At start of line:
        "rowsep"        : r"^==(?!.*==\s*?$)(?=\N*?)",  # == not-heading ws-excl-nl
        "continuation"  : r"^(\N*)\.\.(?!\.)(?=\N)",    # .. ws-excl-nl or .. not-dot

        # Within text:
        "columnsep"     : r"\|\|(?!\|)(?=\N)",          # || ws-excl-nl or || not-pipe
        })

    patterns = get_patterns(syntax)



    # Pattern details.

    table_region_pattern_names = MoinParser.region_pattern_names + [
        "columnsep", "continuation", "regionend", "rowsep",
        ]



    # Pattern handlers.

    handlers = {}
    handlers.update(MoinParser.handlers)
    handlers.update({
        "columnsep" : MoinParser.end_region,
        "continuation" : parse_continuation,
        "rowsep" : MoinParser.end_region,
        "regionend" : parse_table_end,
        })

parser = TableParser

# vim: tabstop=4 expandtab shiftwidth=4
